@@ -42,6 +42,7 @@ static gint gwget_data_process_information (GwgetData *gwgetdata);
 static void gwget_data_update_statistics_ui(GwgetData *gwgetdata);
 static void convert_time2str(gchar *buffer, guint32 time);
 static void gwget_gnotify_finished(GwgetData *gwgetdata);
+static void gwget_download_finished(GwgetData *gwgetdata);
 
 static void
 convert_time2str (gchar *buffer, guint32 time)
@@ -50,13 +51,18 @@ convert_time2str (gchar *buffer, guint32 time)
 	     (gint)((time % 3600) / 60), (gint)(time % 60));
 }
 
+void
+gwget_data_set_state_no_sync (GwgetData *gwgetdata, DlState state) 
+{
+	gwgetdata->state=state;
+	gwget_data_update_statistics_ui(gwgetdata);
+}
 
 
 void
 gwget_data_set_state (GwgetData *gwgetdata, DlState state) 
 {
-	gwgetdata->state=state;
-	gwget_data_update_statistics_ui(gwgetdata);
+	gwget_data_set_state_no_sync(gwgetdata, state);
 	gwget_remember_downloads();
 }
 
@@ -212,7 +218,7 @@ gwget_data_update_statistics (GwgetData *gwgetdata)
 			   -1);
 		
 	gwget_data_update_statistics_ui(gwgetdata);
-	if (gwgetdata == gwget_data_get_selected()) {
+	if (gwgetdata == gwget_data_get_selected() || count_download_in_progress() == 1 ) {
 		gtk_window_set_title(GTK_WINDOW(glade_xml_get_widget(xml, "main_window")), title);
 	}
 
@@ -223,8 +229,6 @@ gwget_data_update_statistics (GwgetData *gwgetdata)
 static void
 gwget_data_update_statistics_ui(GwgetData *gwgetdata)
 {
-	gchar *tooltip_message;
-	
 	switch(gwgetdata->state) {
 		case DL_WAITING: gwgetdata->state_str = g_strdup(_("Waiting"));
 				 break;
@@ -241,14 +245,6 @@ gwget_data_update_statistics_ui(GwgetData *gwgetdata)
 		case DL_COMPLETED: gwgetdata->state_str=g_strdup(_("Completed"));
 				   gtk_list_store_set(GTK_LIST_STORE(model),&gwgetdata->file_list,
 							      PERCENTAGE_COLUMN,100,-1);
-				   if (gwget_pref.docked) {
-					   tooltip_message=g_strdup_printf(_("Download Completed\n(%s)"), 
-							   gwgetdata->url);
-					   gtk_tooltips_set_tip (GTK_TOOLTIPS(tray_tooltip), GTK_WIDGET(tray_icon),tooltip_message,NULL);
-					   egg_tray_icon_send_message(tray_icon,100000,tooltip_message,strlen(tooltip_message));
-					   set_icon_idle();
-				   }
-					   
 				   break;
 		case DL_ERROR: 	gwgetdata->state_str = g_strdup_printf("%s: %s",_("Error"),gwgetdata->error_msg);
 				break;
@@ -290,7 +286,7 @@ gwget_data_process_information (GwgetData *gwgetdata)
 		else if (WIFEXITED (status)) {
             		if (WEXITSTATUS (status) == 0) {
                 		gwget_data_set_state (gwgetdata, DL_COMPLETED);
-                     gwget_gnotify_finished(gwgetdata);
+                     		gwget_download_finished(gwgetdata);
 				start_first_waiting_download();
 				if (gwget_pref.open_after_dl) {
 					gwget_data_exec(gwgetdata);
@@ -333,17 +329,18 @@ gwget_data_start_download(GwgetData *gwgetdata)
 	pid_t pid;
 	gint pipe_fd[2];
 	gint number;
-
+	gchar *user_password;
+	user_password=g_strdup("");
 	/* Control the number of download allowed */
 	number=count_download_in_progress();
 
 	if (gwget_pref.limit_simultaneousdownloads==TRUE && number>=gwget_pref.max_simultaneousdownloads) {
-		gwget_data_set_state(gwgetdata,DL_WAITING);
+		gwget_data_set_state_no_sync(gwgetdata,DL_WAITING);
 		return;
 	}
 
 	/* Put the not connected state before */
-	gwget_data_set_state(gwgetdata,DL_NOT_CONNECTED);
+	gwget_data_set_state_no_sync(gwgetdata,DL_NOT_CONNECTED);
 	gwget_data_update_statistics_ui(gwgetdata);
 	
 	/* First check if we are not starting an already started download */
@@ -438,17 +435,27 @@ gwget_data_start_download(GwgetData *gwgetdata)
 			if ( strcmp (gwget_pref.network_mode, "manual") == 0 && 
 			     gwget_pref.http_proxy ) 
 			{
-				setenv("http_proxy",g_strconcat(g_strdup_printf("%s",gwget_pref.http_proxy),":",g_strdup_printf("%d",gwget_pref.http_proxy_port),NULL),1);
-				setenv("ftp_proxy",g_strconcat(g_strdup_printf("%s",gwget_pref.http_proxy),":",g_strdup_printf("%d",gwget_pref.http_proxy_port),NULL),1);
+				if(gwget_pref.proxy_uses_auth) 
+				{
+					user_password=g_strdup_printf("%s:%s@",gwget_pref.proxy_user,gwget_pref.proxy_password);
+				}
+
+				setenv("http_proxy", g_strconcat("http://",g_strdup_printf(user_password),g_strdup_printf("%s",gwget_pref.http_proxy),":",g_strdup_printf("%d",gwget_pref.http_proxy_port),NULL), 1);
+				setenv("ftp_proxy", g_strconcat("http://",g_strdup_printf(user_password),g_strdup_printf("%s",gwget_pref.http_proxy),":",g_strdup_printf("%d",gwget_pref.http_proxy_port),NULL), 1);
 		 	        argv[arg]="-Yon";
 				arg++;
 			}
 			
 			if ( strcmp (gwget_pref.network_mode, "default" ) == 0 && gwget_pref.gnome_http_proxy && gwget_pref.gnome_use_proxy) 
 			{
-				setenv("http_proxy",g_strconcat(g_strdup_printf("%s",gwget_pref.gnome_http_proxy),":",g_strdup_printf("%d",gwget_pref.gnome_http_proxy_port),NULL),1);
-				setenv("ftp_proxy",g_strconcat(g_strdup_printf("%s",gwget_pref.http_proxy),":",g_strdup_printf("%d",gwget_pref.http_proxy_port),NULL),1);
-	            		argv[arg]="-Yon";
+				if(gwget_pref.gnome_proxy_uses_auth) 
+				{
+					user_password=g_strdup_printf("%s:%s@",gwget_pref.gnome_proxy_user,gwget_pref.gnome_proxy_password);
+				}
+
+				setenv("http_proxy",g_strconcat("http://",g_strdup_printf(user_password),g_strdup_printf("%s",gwget_pref.gnome_http_proxy),":",g_strdup_printf("%d",gwget_pref.gnome_http_proxy_port),NULL),1);
+				setenv("ftp_proxy",g_strconcat("http://",g_strdup_printf(user_password),g_strdup_printf("%s",gwget_pref.gnome_http_proxy),":",g_strdup_printf("%d",gwget_pref.gnome_http_proxy_port),NULL),1);
+		 	        argv[arg]="-Yon";
 				arg++;
 			}
 			
@@ -496,6 +503,13 @@ gwget_data_set_total_size (GwgetData *gwgetdata, guint64 total_size)
 	gwgetdata->total_size = total_size;
 }
 
+GwgetData *
+gwget_data_new (gchar *url)
+{
+
+	return gwget_data_create (url, gwget_pref.download_dir);
+
+}
 
 GwgetData * 
 gwget_data_create(gchar *url, gchar *dir)
@@ -566,7 +580,6 @@ gwget_data_set_filename(GwgetData* gwgetdata,gchar *str)
 	gwgetdata->local_filename = g_strconcat (gwgetdata->dir, 
 			                         str,
 						 NULL);
-	gwget_remember_downloads();
 }
 
 /* Return the gwgetdata that is selected in the treeview */
@@ -599,7 +612,7 @@ void gwget_data_stop_download(GwgetData *data)
 	
 	if (gwget_data_run(data)) {
 		/* Kill wget process */
-		kill (data->wget_pid, SIGINT);
+		kill (data->wget_pid, SIGKILL);
 		
 		/* Remove callback that communicates with wget */
 		gtk_timeout_remove (data->log_tag);
@@ -660,8 +673,6 @@ gwget_data_set_filename_from_url(GwgetData *gwgetdata,gchar *url)
 	} else {
 		gwgetdata->filename = g_strdup(filename);
 	}
-
-	gwget_remember_downloads();
 }
 
 /* Add a gwgetdata to the main window */
@@ -709,8 +720,8 @@ gwget_data_add_download(GwgetData *gwgetdata)
 		}
 	}
 	downloads = g_list_append(downloads,gwgetdata);
-	gwget_data_set_state(gwgetdata,DL_NOT_CONNECTED);
 	new_download(gwgetdata);
+	gwget_data_set_state(gwgetdata,DL_NOT_CONNECTED);
 }
 
 void
@@ -722,7 +733,7 @@ gwget_data_exec (GwgetData *gwgetdata)
 	uri = gnome_vfs_make_uri_from_input_with_dirs (gwgetdata->local_filename,
 							GNOME_VFS_MAKE_URI_DIR_CURRENT);
 	if (!gnome_url_show (uri, &err)) {
-		run_dialog(_("Error opening file"),_("Couldn't open the file"));
+		run_dialog_error(_("Error opening file"), _("Couldn't open the file"));
 	}
 }
 
@@ -763,46 +774,64 @@ gwget_gnotify_finished(GwgetData *gwgetdata) {
    g_free(app_msg);
    g_free(icon_msg);
    g_free(fn_msg);
-	g_free(gwgetdata->icon_name);
+}
+
+
+static void
+gwget_download_finished (GwgetData *gwgetdata)
+{
+   gwget_gnotify_finished(gwgetdata);
+   gwget_tray_notify (_("Download Complete"), gwgetdata->filename, gwgetdata->icon_name);
+   gtk_widget_set_sensitive(glade_xml_get_widget(xml, "clear_button"), TRUE);
 }
 
 void
 gwget_data_set_menus (GwgetData *gwgetdata)
 {
 
-
 	if (gwgetdata!=NULL) {
 		if (gwget_data_run(gwgetdata)) {
-			/* Menu items */
 			gtk_widget_set_sensitive(glade_xml_get_widget(xml,"pause_menuitem"),TRUE);
 			gtk_widget_set_sensitive(glade_xml_get_widget(xml,"continue_menuitem"),FALSE);
 			gtk_widget_set_sensitive(glade_xml_get_widget(xml,"cancel_menuitem"),TRUE);
-			/* popup */
-			gtk_widget_set_sensitive(glade_xml_get_widget(xml,"pause_download"),TRUE);
-			gtk_widget_set_sensitive(glade_xml_get_widget(xml,"continue_download"),FALSE);
-			gtk_widget_set_sensitive(glade_xml_get_widget(xml,"cancel_download"),TRUE);
 		} else {
 			if (gwgetdata->state==DL_COMPLETED) {
 				gtk_widget_set_sensitive(glade_xml_get_widget(xml,"continue_menuitem"),FALSE);
 				gtk_widget_set_sensitive(glade_xml_get_widget(xml,"pause_menuitem"),FALSE);
 				gtk_widget_set_sensitive(glade_xml_get_widget(xml,"cancel_menuitem"),FALSE);
-				gtk_widget_set_sensitive(glade_xml_get_widget(xml,"continue_download"),FALSE);
-				gtk_widget_set_sensitive(glade_xml_get_widget(xml,"pause_download"),FALSE);
-				gtk_widget_set_sensitive(glade_xml_get_widget(xml,"cancel_download"),FALSE);
 			} else {
 				gtk_widget_set_sensitive(glade_xml_get_widget(xml,"continue_menuitem"),TRUE);
 				gtk_widget_set_sensitive(glade_xml_get_widget(xml,"pause_menuitem"),FALSE);
-				gtk_widget_set_sensitive(glade_xml_get_widget(xml,"continue_download"),TRUE);
-				gtk_widget_set_sensitive(glade_xml_get_widget(xml,"pause_download"),FALSE);
 			}
 		}
 	} else {
-			gtk_widget_set_sensitive(glade_xml_get_widget(xml,"continue_menuitem"),FALSE);
-			gtk_widget_set_sensitive(glade_xml_get_widget(xml,"pause_menuitem"),FALSE);
-			gtk_widget_set_sensitive(glade_xml_get_widget(xml,"cancel_menuitem"),FALSE);
-			gtk_widget_set_sensitive(glade_xml_get_widget(xml,"continue_download"),FALSE);
-			gtk_widget_set_sensitive(glade_xml_get_widget(xml,"pause_download"),FALSE);
-			gtk_widget_set_sensitive(glade_xml_get_widget(xml,"cancel_download"),FALSE);
-			gtk_widget_set_sensitive(glade_xml_get_widget(xml,"properties_menuitem"),FALSE);
+		gtk_widget_set_sensitive(glade_xml_get_widget(xml,"continue_menuitem"),FALSE);
+		gtk_widget_set_sensitive(glade_xml_get_widget(xml,"pause_menuitem"),FALSE);
+		gtk_widget_set_sensitive(glade_xml_get_widget(xml,"cancel_menuitem"),FALSE);
 	}
 }
+
+void 
+gwget_data_set_popupmenu (GwgetData *gwgetdata)
+{
+	if (gwgetdata!=NULL) {
+		if (gwget_data_run(gwgetdata)) {
+			gtk_widget_set_sensitive(glade_xml_get_widget(xml,"pause_download"),TRUE);
+                        gtk_widget_set_sensitive(glade_xml_get_widget(xml,"continue_download"),FALSE);
+                        gtk_widget_set_sensitive(glade_xml_get_widget(xml,"cancel_download"),TRUE);
+		} else {
+			if (gwgetdata->state==DL_COMPLETED) {
+                                gtk_widget_set_sensitive(glade_xml_get_widget(xml,"continue_download"),FALSE);
+                                gtk_widget_set_sensitive(glade_xml_get_widget(xml,"pause_download"),FALSE);
+                                gtk_widget_set_sensitive(glade_xml_get_widget(xml,"cancel_download"),FALSE);
+                        } else {
+                                gtk_widget_set_sensitive(glade_xml_get_widget(xml,"continue_download"),TRUE);
+                                gtk_widget_set_sensitive(glade_xml_get_widget(xml,"pause_download"),FALSE);
+                        }
+		}
+	} else {
+                gtk_widget_set_sensitive(glade_xml_get_widget(xml,"continue_download"),FALSE);
+                gtk_widget_set_sensitive(glade_xml_get_widget(xml,"pause_download"),FALSE);
+                gtk_widget_set_sensitive(glade_xml_get_widget(xml,"cancel_download"),FALSE);
+        }
+}
