@@ -22,9 +22,10 @@
 #include <libgnome/gnome-url.h>
 #include <libgnome/gnome-program.h>
 #include <libgnome/gnome-init.h>
+#include <libgnomevfs/gnome-vfs.h>
 #include <libgnomevfs/gnome-vfs-utils.h>
 #include <libgnomevfs/gnome-vfs-mime-utils.h>
-#include <gnome-vfs-module-2.0/libgnomevfs/gnome-vfs-mime.h>
+#include <libgnomevfs/gnome-vfs-mime.h>
 
 #include "main_window.h"
 #include "main_window_cb.h"
@@ -32,6 +33,9 @@
 #include "gwget_data.h"
 #include "utils.h"
 #include "systray.h"
+#include "md5.h"
+
+#define MD5BUFSIZE 512
 
 static void inform_limit_speed_change(void);
 
@@ -66,11 +70,11 @@ on_treeview1_button_press_event(GtkWidget *widget, GdkEventButton *event,gpointe
 	if (event->type == GDK_BUTTON_PRESS) {
 		event_button = (GdkEventButton *) event;
 		if (event->button==3 && gtk_tree_selection_get_selected (select, &model, &iter)) {
+			GtkTreeSelection *selection;
+			GtkTreePath *path;
 
 			/* Select right-clicked line */
-			GtkTreeSelection *selection;
 			selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(treev));
-			GtkTreePath *path;
 			if (gtk_tree_view_get_path_at_pos(GTK_TREE_VIEW(treev),
                                              (gint) event->x, 
                                              (gint) event->y,
@@ -81,10 +85,10 @@ on_treeview1_button_press_event(GtkWidget *widget, GdkEventButton *event,gpointe
 				gtk_tree_path_free(path);
 			}
 
-			gwget_data_set_menus(gwget_data_get_selected());
 			popup=glade_xml_get_widget(xml,"popup1");
 			gtk_menu_popup (GTK_MENU(popup), NULL, NULL, NULL, NULL, 
 							event_button->button, event_button->time);
+			gwget_data_set_popupmenu (gwget_data_get_selected());
 			return TRUE;
 		}
 	}
@@ -176,7 +180,6 @@ on_about1_activate(GtkWidget *widget, gpointer data)
 
 	static const gchar *authors[] = { 
 			"David Sedeño Fernández <david@alderia.com>",
-			"Julien Cegarra <julienc@psychologie-fr.org>",
 			NULL
 	};
 		
@@ -187,7 +190,7 @@ on_about1_activate(GtkWidget *widget, gpointer data)
 	gtk_show_about_dialog (NULL,
 			       "name", _("Gwget"), 
 			       "version", VERSION,
-			       "copyright", "Copyright © 2004 David Sedeño Fernández",
+			       "copyright", "Copyright © 2004-2007 David Sedeño Fernández",
 			       "comments", _("Download Manager for GNOME."),
 			       "authors", authors,
 			       "translator-credits", _("translator-credits"),
@@ -272,7 +275,7 @@ void
 on_boton_pref_clicked(GtkWidget *widget, gpointer data)
 {
 	gchar *xml_file = NULL;
-	GtkWidget *window = NULL,*entry=NULL, *checkbutton=NULL;
+	GtkWidget *window = NULL,*entry=NULL, *checkbutton=NULL, *main_window = NULL;
 	
 	if (!xml_pref) {
 		xml_file=g_build_filename(DATADIR,"preferences.glade",NULL);
@@ -280,9 +283,12 @@ on_boton_pref_clicked(GtkWidget *widget, gpointer data)
 		glade_xml_signal_autoconnect(xml_pref);
 	}
 		
+	main_window = glade_xml_get_widget (xml, "main_window");
 	window = glade_xml_get_widget (xml_pref, "pref_window");
 	entry = glade_xml_get_widget(xml_pref,"save_in_entry");
 
+	gtk_window_set_transient_for(GTK_WINDOW(window), GTK_WINDOW(main_window));
+
 	if (gwget_pref.download_dir && strlen(gwget_pref.download_dir)!=0) {
 		gtk_entry_set_text(GTK_ENTRY(entry), gwget_pref.download_dir);
 	} else {
@@ -313,9 +319,9 @@ on_boton_pref_clicked(GtkWidget *widget, gpointer data)
 		gtk_entry_set_text(GTK_ENTRY(entry),gwget_pref.http_proxy);
 		checkbutton=glade_xml_get_widget(GLADE_XML(xml_pref),"proxy_uses_auth_radio");
 		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(checkbutton),gwget_pref.proxy_uses_auth);
-		entry = glade_xml_get_widget(xml_pref,"proxy_user_entry");
+		entry = glade_xml_get_widget(xml_pref,"proxy_user");
 		gtk_entry_set_text(GTK_ENTRY(entry),gwget_pref.proxy_user);
-		entry = glade_xml_get_widget(xml_pref,"proxy_password_entry");
+		entry = glade_xml_get_widget(xml_pref,"proxy_password");
 		gtk_entry_set_text(GTK_ENTRY(entry),gwget_pref.proxy_password);
 
 	}
@@ -616,7 +622,7 @@ on_cancel_download_activate(GtkWidget *widget,gpointer data)
 	if (gwgetdata) 
 	{
 		msg = g_strdup_printf(_("Really cancel current download?\n(URL: %s)"),gwgetdata->url);
-		response = run_dialog(_("Cancel download?"),_(msg), _("Cancel"));
+		response = run_dialog(_("Delete download?"),_(msg), _("Delete"));
 		if (response==GTK_RESPONSE_OK) {
 			gwget_data_stop_download(gwgetdata); 
 			unlink (gwgetdata->local_filename);
@@ -661,6 +667,8 @@ on_remove_completed_activate(GtkWidget *widget, gpointer data)
 					gtk_tree_model_iter_next(model,&iter);
 				}
 			}
+			gtk_widget_set_sensitive(glade_xml_get_widget(xml, "clear_button"), FALSE);
+			gtk_window_set_title(GTK_WINDOW(glade_xml_get_widget(xml, "main_window")), _("Gwget - Download Manager"));
 		}
 	}
 				
@@ -740,7 +748,7 @@ on_view_toolbar_activate(GtkWidget *widget,gpointer data)
 	GtkWidget *toolbar,*menu_item;
 	gboolean state;
 	
-	toolbar=glade_xml_get_widget(GLADE_XML(xml),"bonobotoolbar");
+	toolbar=glade_xml_get_widget(GLADE_XML(xml),"toolbar1");
 	menu_item=glade_xml_get_widget(GLADE_XML(xml),"view_toolbar");
 	state = gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(menu_item));
 	
@@ -790,10 +798,118 @@ on_properties_activate(GtkWidget *widget, gpointer data)
 		gtk_label_set_text(GTK_LABEL(local_file),gwgetdata->filename);
 		local_dir=glade_xml_get_widget(GLADE_XML(xml),"local_dir");
 		gtk_label_set_text(GTK_LABEL(local_dir),gwgetdata->dir);
-		gtk_widget_show(properties);
+		if (gwgetdata->state==DL_COMPLETED) {
+                   gtk_widget_set_sensitive(glade_xml_get_widget(xml, "compare_md5"), TRUE);
+		} else {
+                   gtk_widget_set_sensitive(glade_xml_get_widget(xml, "compare_md5"), FALSE);
+                }
+                gtk_widget_show(properties);
 	}
 }
 
+void
+on_compare_md5_clicked(GtkWidget *widget, gpointer data)
+{
+   GwgetData *gwgetdata;
+
+   gwgetdata=gwget_data_get_selected();
+	
+   if (gwgetdata) {
+      gtk_widget_show(glade_xml_get_widget(GLADE_XML(xml),"md5_window"));
+   }
+}
+
+void
+on_md5ok_button_clicked(GtkWidget *widget, gpointer data)
+{
+   GtkWidget *md5;
+   GwgetData *gwgetdata;
+   const gchar *entrytext;
+   GtkMessageDialog *dialog;
+   GtkMessageType msg_type;
+
+   gchar *uri;
+   GnomeVFSHandle *fd;
+   GnomeVFSResult result;
+   GString *localfile, *md5res;
+   GString *msg;
+   const gchar *err;
+   gint i;
+   GnomeVFSFileSize fsize;
+   
+   struct md5_ctx md5context;
+   guchar md5digest[16];
+   gchar dataread[MD5BUFSIZE];
+   gboolean close_md5_dlg;
+  
+   gwgetdata = gwget_data_get_selected();
+	
+   if (gwgetdata) {
+      md5=glade_xml_get_widget(GLADE_XML(xml),"md5_window");
+      entrytext=gtk_entry_get_text(GTK_ENTRY(glade_xml_get_widget(GLADE_XML(xml),"md5_entry")));
+      localfile=g_string_new(gwgetdata->dir);
+      localfile=g_string_append(localfile, gwgetdata->filename); 
+      uri=gnome_vfs_get_uri_from_local_path(localfile->str);
+      result=gnome_vfs_open(&fd, uri, GNOME_VFS_OPEN_READ);
+
+      if (result != GNOME_VFS_OK) {
+         err=gnome_vfs_result_to_string(result);
+         g_printerr(_("error %d when accessing %s: %s\n"), result, uri, err);
+         return;
+      }
+
+      md5_init_ctx(&md5context);
+
+      while(result == GNOME_VFS_OK) {
+         result=gnome_vfs_read(fd, dataread, MD5BUFSIZE, &fsize);  
+         md5_process_bytes(dataread, fsize, &md5context);
+         if (fsize < MD5BUFSIZE)
+            break;
+      }
+
+      md5_finish_ctx(&md5context, md5digest);
+
+      result=gnome_vfs_close(fd);
+      if (result != GNOME_VFS_OK) {
+         err=gnome_vfs_result_to_string(result);
+         g_printerr(_("error %d when accessing %s: %s\n"), result, uri, err);
+         return;
+      }
+
+      md5res=g_string_new("");
+      for (i = 0; i < 16; i++) {
+         g_string_append_printf(md5res,"%02x", md5digest[i]);
+      }
+
+      msg=g_string_new("");
+      if (g_str_equal(md5res->str, entrytext)) {
+         close_md5_dlg=TRUE;
+         g_string_append_printf(msg, _("<span size=\"large\" weight=\"bold\">MD5SUM Check PASSED!</span>"));
+         msg_type=GTK_MESSAGE_INFO;
+      } else {
+         g_string_append_printf(msg, _("<span size=\"large\" weight=\"bold\">MD5SUM Check FAILED!</span>"));
+         close_md5_dlg=FALSE;
+         msg_type=GTK_MESSAGE_WARNING;
+      }
+
+      dialog=(GtkMessageDialog *)gtk_message_dialog_new_with_markup(GTK_WINDOW(md5),
+                                                     GTK_DIALOG_DESTROY_WITH_PARENT,
+                                                     msg_type,
+                                                     GTK_BUTTONS_CLOSE,
+                                                     msg->str);
+                                  
+      gtk_dialog_run(GTK_DIALOG(dialog));
+      gtk_widget_destroy(GTK_WIDGET(dialog));   
+      g_free(uri); 
+      g_string_free(localfile, TRUE);
+      g_string_free(md5res, TRUE);
+      g_string_free(msg, TRUE);
+      if (close_md5_dlg) {
+         gtk_widget_hide(GTK_WIDGET(md5));
+      }
+   }
+}
+
 void 
 on_check_actual_size_toggled(GtkWidget *widget, gpointer data)
 {
